/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/

#include "common.h"

/*-------------------------------------------------------
					General Functions
-------------------------------------------------------*/
GLvoid CGl::ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{

	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(90.0f,(GLfloat)width/(GLfloat)height,0.1f,2000.0f);
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

}								


int CGl::InitGL(GLvoid)								// All our OpenGL inits go here
{
	//Reset all
	glViewport(0,0,Width,Height);						
	glMatrixMode(GL_PROJECTION);						
	glLoadIdentity();									
	// Calculate The Aspect Ratio Of The Window
	gluPerspective(90.0f,(GLfloat)Width/(GLfloat)Height,0.1f,2000.0f);
	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();									

	//General
	glEnable(GL_TEXTURE_2D);						// Enable Texture Mapping 
	glShadeModel(GL_FLAT);						// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);					// Black Background
	glClearDepth(10.0f);							// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);						// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);							// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Really Nice Perspective Calculations
	glColor4f(1.0f,1.0f,1.0f,1.0f);			// Full Brightness, 50% Alpha ( NEW )
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);		// Blending Function For Translucency Based On Source Alpha Value ( NEW )
	SDL_ShowCursor(FALSE);								// Dont Show Mouse Pointer

	//Lighting wouldnt work too well with this game
	
	return TRUE;								// Initialization Went fine
}


GLvoid CGl::KillGLWindow(GLvoid)								// Properly Kill window
{
	SDL_Quit( );
	SDL_ShowCursor(TRUE);								// Show Mouse Pointer
}

 
int CGl::CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	
	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag
	int val;
    if ( SDL_InitSubSystem( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Fetch the video info */
    videoInfo = (SDL_VideoInfo *)SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* the flags to pass to SDL_SetVideoMode                            */
    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL          */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering       */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    videoFlags |= SDL_RESIZABLE;       /* Enable window resizing        */
	if(fullscreen)
		videoFlags |= SDL_FULLSCREEN;
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;


    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_GetAttribute(SDL_GL_DOUBLEBUFFER, &val);

    /* get a SDL surface */
    surface = SDL_SetVideoMode( width, height, bits,videoFlags );

    /* Verify there is a surface */
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}
	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
	    fprintf( stderr, "Could not initialize OpenGL.\n" );
	    Quit( 1 );								// Return FALSE
	}

	return 1;									// Success
}
/*Just some primitive drawing functions*/
void DrawQuad(GLfloat x,GLfloat y,GLfloat w,GLfloat h,CIndivTexture *Text){
	int Texture = TRUE;
	if(!Text) Texture = FALSE;
	if(Texture){
		Text->BindTexture();
		glBegin(GL_QUADS);					// Start Drawing A Quad
		glTexCoord2f(0.0f,1.0f);	glVertex2f(x,   y  );			// Top Left Of The Quad
		glTexCoord2f(1.0f,1.0f);	glVertex2f(x+w, y  );			// Top Right Of The Quad
		glTexCoord2f(1.0f,0.0f);	glVertex2f(x+w, y-h);			// Bottom Right Of The Quad
		glTexCoord2f(0.0f,0.0f);	glVertex2f(x,   y-h);			// Bottom Left Of The Quad
		glEnd();						// Done Drawing The Quad
	}
	else
	{
		glBegin(GL_QUADS);					// Start Drawing A Quad
		glVertex2f(x,   y  );			// Top Left Of The Quad
		glVertex2f(x+w, y  );			// Top Right Of The Quad
		glVertex2f(x+w, y-h);			// Bottom Right Of The Quad
		glVertex2f(x,   y-h);			// Bottom Left Of The Quad
		glEnd();						// Done Drawing The Quad
	}
}

void DrawQuad2d(GLint x,GLint y,GLint w,GLint h,CIndivTexture *Text){
	int Texture = TRUE;
	if(!Text) Texture = FALSE;
	if(Texture){
		Text->BindTexture();
		glBegin(GL_QUADS);					// Start Drawing A Quad
		glTexCoord2d(0,1);	glVertex2d(x,   y  );			// Top Left Of The Quad
		glTexCoord2d(1,1);	glVertex2d(x+w, y  );			// Top Right Of The Quad
		glTexCoord2d(1,0);	glVertex2d(x+w, y-h);			// Bottom Right Of The Quad
		glTexCoord2d(0,0);	glVertex2d(x,   y-h);			// Bottom Left Of The Quad
		glEnd();						// Done Drawing The Quad
	}
	else
	{
		glBegin(GL_QUADS);					// Start Drawing A Quad
		glVertex2d(x,   y  );			// Top Left Of The Quad
		glVertex2d(x+w, y  );			// Top Right Of The Quad
		glVertex2d(x+w, y-h);			// Bottom Right Of The Quad
		glVertex2d(x,   y-h);			// Bottom Left Of The Quad
		glEnd();						// Done Drawing The Quad
	}
}
//Make a cube
void DrawCube(GLfloat x,GLfloat y,GLfloat z,GLfloat w,GLfloat h,GLfloat v)
{	
	v *= -1; // Right way
	glBegin(GL_QUADS);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Forward
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x, y+h,  z);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( x+w, y+h,  z);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( x+w,  y,  z);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,  y,  z);	// Top Left Of The Texture and Quad
		// Back Face
		glNormal3f( 0.0f, 0.0f, -1.0f);			// Normal Pointing Back

		glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y+h, z+v);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y, z+v);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( x+w,  y, z+v);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( x+w, y+h, z+v);	// Bottom Left Of The Texture and Quad
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);			// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,  y, z+v);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,  y,  z);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( x+w,  y,  z);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( x+w,  y, z+v);	// Top Right Of The Texture and Quad
		// Bottom Face
		glNormal3f( 0.0f, -1.0f, 0.0f);			// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x, y+h, z+v);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( x+w, y+h, z+v);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( x+w, y+h,  z);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y+h,  z);	// Bottom Right Of The Texture and Quad
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);			// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( x+w, y+h, z+v);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( x+w,  y, z+v);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f( x+w,  y,  z);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f( x+w, y+h,  z);	// Bottom Left Of The Texture and Quad
		// Left Face
		glNormal3f( -1.0f, 0.0f, 0.0f);			// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x, y+h, z+v);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y+h,  z);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,  y,  z);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,  y, z+v);	// Top Left Of The Texture and Quad
	glEnd();
}
	



