/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/
 #include "common.h"

extern CLog Log;
/*-------------------------------------------------------
					Texture Functions
-------------------------------------------------------*/
extern int TexCount;
extern GLuint Tlist[100];
extern CTexture CTexture;

int CIndivTexture::LoadGLTexture(char *file)								// Load Bitmaps And Convert To Textures
{
	int Status=FALSE;							// Status Indicator
	SDL_Surface *TextureImage;
    TextureFilename = new char [strlen(file)+strlen(DataDir)+2];
    sprintf(TextureFilename,"%s%s",DataDir,file);
	TextureImage = SDL_LoadBMP( TextureFilename );
	Plog(&Log,"Flipping %s\n",file);
	TextureImage = Flip(TextureImage);	//Flip it for SDL
	//Convert it To RGB from BGR
	Plog(&Log,"Converting %s\n",file);
	TextureImage = MakeRGB(TextureImage);

	if (TextureImage)
	{
		Status=TRUE;							// Set The Status To TRUE

		Plog(&Log,"%s--Filename\n\n",TextureFilename);
		glGenTextures(1,&this->TexID);
		// Create MipMapped Texture
		this->BindTexture();
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage->w,
		TextureImage->h, 0, GL_RGB, GL_UNSIGNED_BYTE,
		TextureImage->pixels);


	}
	else{
		Plog(&Log,"%d -- Error no. for getting texture\n",glGetError());
	}

	if ( TextureImage )
		    SDL_FreeSurface( TextureImage );
	return Status;								// Return The Status
}

SDL_Surface *Flip(SDL_Surface *src)
{
	int y;
	SDL_Surface *b = SDL_CreateRGBSurface(src->flags,
										  src->w,
										  src->h,
										  src->format->BitsPerPixel,
										  src->format->Rmask,
										  src->format->Gmask,
										  src->format->Bmask,
										  src->format->Amask);

	SDL_Rect sr,dr;
	sr.x = 0;
	sr.y = 0;
	sr.w = src->w;
	sr.h = 1;
	dr.x = 0;
	dr.y = src->h;
	dr.w = src->w;
	dr.h = 1;
	
	for(y = 0;y<src->h;y++){
		if(SDL_BlitSurface(src, &sr, b, &dr) != 0)//Use blit so we dont need to know what kind of pixels it has
			printf("Didnt work %d",y);
			sr.y++;
			dr.y--;
	}

	return b;
}
SDL_Surface *MakeRGB(SDL_Surface *src)
{
	int x,y;
	Uint8 r,g,b;
	Uint32 p;
	SDL_Surface *s = SDL_CreateRGBSurface(src->flags,
										  src->w,
										  src->h,
										  src->format->BitsPerPixel,
										  src->format->Rmask,
										  src->format->Gmask,
										  src->format->Bmask,
										  src->format->Amask);
	SDL_LockSurface(s);
	SDL_LockSurface(src);
	for(y = 0;y<src->h;y++){
		for(x = 0;x<src->w;x++){
			p = getpixel(src,x,y);
			SDL_GetRGB(p, src->format,&r,&g,&b);
			
			p = SDL_MapRGB(src->format, b,g,r);
			putpixel(s,x,y,p);
		}
	}
	SDL_UnlockSurface(s);
	SDL_UnlockSurface(src);
	return s;
}
Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}


void CIndivTexture::BindTexture(void){
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,TexID);
}

CIndivTexture::~CIndivTexture(){
}

CTexture::CTexture(){
}
CTexture::~CTexture(){
}
